/*
   FILNAME:       CTRL.H

   PURPOSE:   

   CREATED BY:    Per Ghosh
   ENVIRONMENT:	Visual C++


INFO:
*/

#ifndef __CTRL_H_
#define __CTRL_H_ 

extern BOOL pg_RegisterWindow( WNDCLASS& wndclassWindow );

/*-------------------------------------------------------------------------------------------------
Name:	   DECLARE_REGISTER_WINDOW()
Type:    macro
Purpose: declare some function for custom controls
*/
#define DECLARE_REGISTER_WINDOW()                                                              \
public:                                                                                        \
	static BOOL pg_RegisterClass();                                                             \
protected:                                                                                     \
	static LRESULT CALLBACK pg_WndClassProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam); \
   static BOOL m_bRegistered;                                                                  \
	virtual void PostNcDestroy();                                                               


/*-------------------------------------------------------------------------------------------------
Name:	   IMPLEMENT_REGISTER_WINDOW
Type:    macro
Purpose: declare functions for custom controls

Param:   class_name = name of the class (C++ class name)
         style      = styles for the window
         icon       = identifies icon to the window
         background = the window background

Throw:   CResourceException, if failing to register window
*/
#define IMPLEMENT_REGISTER_WINDOW(pg_class_name, pg_dwStyle, pg_hIcon, pg_hCursor, pg_hBackground)\
BOOL pg_class_name::pg_RegisterClass()                                                         \
{                                                                                              \
	WNDCLASS wndclass;                                                                          \
   LPCTSTR  pszName       = #pg_class_name;                                                    \
                                                                                               \
	wndclass.style         = pg_dwStyle;                                                        \
	wndclass.lpfnWndProc   = pg_class_name::pg_WndClassProc;                                    \
	wndclass.hIcon         = (HICON)pg_hIcon;                                                   \
	wndclass.hCursor       = (HCURSOR)pg_hCursor;                                               \
	wndclass.hbrBackground = (HBRUSH)pg_hBackground;                                            \
   wndclass.lpszClassName = pszName;                                                           \
                                                                                               \
   return pg_RegisterWindow( wndclass );                                                       \
}                                                                                              \
                                                                                               \
                                                                                               \
LRESULT CALLBACK pg_class_name::pg_WndClassProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)\
{                                                                                              \
	pg_class_name* pWnd = new pg_class_name();                                                  \
	pWnd->Attach(hWnd);                                                                         \
   ::SetWindowLong(hWnd, GWL_WNDPROC, (DWORD)AfxWndProc/*MFC window proc*/);                   \
	return ::CallWindowProc(AfxWndProc/*MFC window proc*/, hWnd, msg, wParam, lParam);          \
}                                                                                              \
                                                                                               \
void pg_class_name::PostNcDestroy()                                                            \
{                                                                                              \
   delete this;                                                                                \
}                                                                                              \
                                                                                               \
BOOL pg_class_name::m_bRegistered = pg_class_name::pg_RegisterClass();     


#endif             
