/*
   FILNAME:       ctrlLBStrikeOut.cpp

   PURPOSE:       A control that has the ability to disable and draw a line across for items

   CREATED BY:    Per Ghosh
   ENVIRONMENT:	Visual C++


INFO:
*/


#include "stdafx.h"
#include "control.h"
#include "ctrlLBStrikeOut.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/************************************************
   *   Constructor/Destructor 
   *
 * * *
  ***
   *
*/     

//-------------------------------------------------------------------------------------------------
CctrlLBStrikeOut::CctrlLBStrikeOut() :
   m_dwFlags(0), m_pFont(NULL), m_chStrikeClue('\0'), m_iSelectedItem(-1)
{
      m_colItem = m_colStrikeItem = ::GetSysColor( COLOR_WINDOW );
      m_dwFlags = CTRLLBSTRIKEOUT_UNABLESELECTSTRIKEITEM;
}

//-------------------------------------------------------------------------------------------------
CctrlLBStrikeOut::~CctrlLBStrikeOut()
{
   DeleteResource( CTRLLBSTRIKEOUT_FONT );
}

/************************************************
   *   Virtual functions
   *
 * * *
  ***
   *
*/

/*-------------------------------------------------------------------------------------------------
Name:	   OnChildNotify()
Type:    virtual function
Purpose: Process some notify messages here

Param:   message  = message type
         wParam   = depends on message
         lParam   = depends on message

Return:  BOOL, FALSE if parent should handle the message to.
*/
BOOL CctrlLBStrikeOut::OnChildNotify(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* plResult) 
{
	switch(message)
	{
	case WM_DRAWITEM:
		DrawItem((LPDRAWITEMSTRUCT)lParam);
      return TRUE;
		break;

   case WM_COMMAND :
      switch( HIWORD(wParam) )
      {
      case LBN_SELCHANGE :
         OnLBSOSelChange();
         break;
      }
	}
	return CListBox::OnChildNotify(message, wParam, lParam, plResult);
}


/************************************************
   *   Get/Set
   *
 * * *
  ***
   *
*/

void CctrlLBStrikeOut::SetStrikeClue( char chClue )
{
   if( chClue )
   {
      m_chStrikeClue =  chClue;
      m_dwFlags      |= CTRLLBSTRIKEOUT_CHARFORSTRIKE;
   }
   else
   {
      m_chStrikeClue =  chClue;
      m_dwFlags      &= ~CTRLLBSTRIKEOUT_CHARFORSTRIKE;
   }
}

void CctrlLBStrikeOut::SetStrikeFlag( BOOL bFlag /*=TRUE*/ )
{
   if( bFlag )
   {
      m_dwFlags |= CTRLLBSTRIKEOUT_FLAGFORSTRIKE;
   }
   else
   {
      m_dwFlags &= ~CTRLLBSTRIKEOUT_FLAGFORSTRIKE;
   }
}


/************************************************
   *   Functions 
   *
 * * *
  ***
   *
*/
BOOL CctrlLBStrikeOut::Create( DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT uId )
{
   ASSERT( dwStyle & LBS_OWNERDRAWFIXED );

	return CListBox::Create(dwStyle, rect, pParentWnd, uId);
}

/*-------------------------------------------------------------------------------------------------
Name:	   CreateDefaultResource()
Type:    Function
Purpose: Creates default resources for the control
*/
void CctrlLBStrikeOut::CreateDefaultResource()
{
   HGDIOBJ hgdiobject;

   m_pFont = new CFont();
   hgdiobject = ::GetStockObject( DEFAULT_GUI_FONT );
   m_pFont->Attach( hgdiobject );

   m_colItem = ::GetSysColor( COLOR_WINDOWTEXT );

   m_colStrikeItem = ::GetSysColor( COLOR_GRAYTEXT );

   m_dwFlags |= (CTRLLBSTRIKEOUT_FONT |
                 CTRLLBSTRIKEOUT_ITEMCOLOR |
                 CTRLLBSTRIKEOUT_STRIKEITEMCOLOR);
}

/*-------------------------------------------------------------------------------------------------
Name:	   StrikeItem()
Type:    Function
Purpose: Strike or unstrike an item

Param:   uItem   = the item that are going to be affected
         bStrike = if we are going to strike or unstrike item

Return:  item or LB_ERR
*/
#define StrikeItem_REDRAW_ITEM   0x00000001
#define StrikeItem_UNSELECT_ITEM 0x00000002
int CctrlLBStrikeOut::StrikeItem( UINT uItem, BOOL bStrike /*=TRUE*/ )
{
   ASSERT( (m_dwFlags & CTRLLBSTRIKEOUT_FLAGFORSTRIKE) || 
           (m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE) );

   char* pszItemText;  // point to itemtext
   DWORD dwItemData;   // item data

   RECT  rect;         // rect thet has to be repainted if changed
   UINT  uOperationItem;  // marks if item should be redrawn

   uOperationItem = 0;                                     // start with no redraw

   if( m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE )         // char marks strike item
   {
      // get text length
      dwItemData = ::SendMessage( GetSafeHwnd(), LB_GETTEXTLEN, (WPARAM)uItem, 0 );

      if( (long)dwItemData == LB_ERR ) return LB_ERR;      // fail if LB_ERR and return

      pszItemText = new char[dwItemData + 2];              // allocate space for text

      // get text
      ::SendMessage( GetSafeHwnd(), LB_GETTEXT, (WPARAM)uItem, (LPARAM)&pszItemText[1] );
      
      if( bStrike )                                        // if striking
      {
         if( pszItemText[1] != m_chStrikeClue )            // is item allready striked
         {
            // remove text
            ::SendMessage( GetSafeHwnd(), LB_DELETESTRING, (WPARAM)uItem, (LPARAM)pszItemText );

            pszItemText[0] = m_chStrikeClue;               // set strike clue (a sign)
            ::SendMessage( GetSafeHwnd(), LB_INSERTSTRING, (WPARAM)uItem, (LPARAM)pszItemText );
            // redraw and unselect if selected
            uOperationItem = StrikeItem_REDRAW_ITEM | StrikeItem_UNSELECT_ITEM;       
         }
      }
      else                                                 // unstrike
      {
         if( pszItemText[1] == m_chStrikeClue )            // if item is striked
         {
            ::SendMessage( GetSafeHwnd(), LB_INSERTSTRING, (WPARAM)uItem, (LPARAM)&pszItemText[2] );
            uOperationItem = StrikeItem_REDRAW_ITEM;       // redraw
         }
      }// endif

      delete pszItemText;                                  // delete buffer
   }
   else // ! ( m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE )
   {
      dwItemData = ::SendMessage( GetSafeHwnd(), LB_GETITEMDATA, (WPARAM)uItem, 0 );

      if( (long)dwItemData == LB_ERR ) return LB_ERR;      // fail if LB_ERR and return

      if( bStrike )                                        // if strike
      {
         if( !(dwItemData & CTRLLBSTRIKEOUT_STRIKE_FLAG) ) // is item allready striked
         {
            dwItemData |= CTRLLBSTRIKEOUT_STRIKE_FLAG;     // set strike flag
            // set new data
            ::SendMessage( GetSafeHwnd(), LB_SETITEMDATA, (WPARAM)uItem, (LPARAM)dwItemData );
            // redraw and unselect if selected
            uOperationItem = StrikeItem_REDRAW_ITEM | StrikeItem_UNSELECT_ITEM;       
         }
      }
      else                                                 // unstrike
      {
         if( dwItemData & CTRLLBSTRIKEOUT_STRIKE_FLAG )    // if item is striked
         {
            dwItemData &= ~CTRLLBSTRIKEOUT_STRIKE_FLAG;    // zero flag
            ::SendMessage( GetSafeHwnd(), LB_SETITEMDATA, (WPARAM)uItem, (LPARAM)dwItemData );
            uOperationItem = StrikeItem_REDRAW_ITEM;       // redraw
         }
      }// endif
   }// end if( m_dwFlags & CTRLLBSTRIKEOUT_CHAR...


   if( uOperationItem & StrikeItem_REDRAW_ITEM )           // redraw ?
   {
      GetItemRect( (int)uItem, &rect );                    // get redraw rect
   	InvalidateRect( &rect, TRUE );                       // invalidate rect

      if( (uOperationItem & StrikeItem_UNSELECT_ITEM) &&
          (m_iSelectedItem == (int)uItem ) )
      {
         m_iSelectedItem = -1;
         SetCurSel( m_iSelectedItem );
      }
   }

   return (int)uItem;                                      // return item index
}

/*-------------------------------------------------------------------------------------------------
Name:	   IsItemStriked()
Type:    Function
Purpose: Tests if item is striked

Param:   uItem = index for item to be tested

Return:  int, TRUE if striked, FALSE if not striked and don't exist LB_ERR
*/
int CctrlLBStrikeOut::IsItemStriked( UINT uItem )
{
   ASSERT( uItem < 0xff000000 );                           // Realistic

   DWORD dwItemData;                                       // Itemdata
   char  pszItemText[CTRLLBSTRIKEOUT_MAXTEXTSIZE];         // buffer for text

   if( m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE )
   {
      // get text and if no item exist for index return LB_ERR
      if( ::SendMessage( GetSafeHwnd(), LB_GETTEXT, (WPARAM)uItem, (LPARAM)pszItemText ) 
          == LB_ERR )
      {
         return LB_ERR;
      }

      if( *pszItemText == m_chStrikeClue ) return TRUE;
   }
   else if( m_dwFlags & CTRLLBSTRIKEOUT_FLAGFORSTRIKE )
   {
      dwItemData = GetItemData( (int)uItem );

      if( dwItemData == LB_ERR ) return LB_ERR;            // if item don't exist

      if( dwItemData & CTRLLBSTRIKEOUT_STRIKE_FLAG ) return TRUE;
   }

   return FALSE;
}

/*-------------------------------------------------------------------------------------------------
Name:	   DrawItem()
Type:    reflected message
Purpose: draw item in the control

Param:   pDIS = pointer to a structure that has information about the control and item
*/
void CctrlLBStrikeOut::DrawItem(LPDRAWITEMSTRUCT pDIS) 
{
   CDC     dc;           // DC to draw on
   CFont*  pfontOld;     // Pointer to old font
   CBrush  brush;        // Brush for different drawingmethods

   BOOL    bStrikeItem;

   UINT    uTextLength;  // Text length for current item
   char*   pszItemText;  // Pointer to buffer where the text are copied to
   char*   pszShowText;  // Pointer to text that are to br drawn

   COLORREF colorref;    // Color for different drawing functions

   CSize   size;         // Text size
   POINT   point[2];     // Points for strike line

   dc.Attach( pDIS->hDC );                                 // attach dc to control window
                                                            
   bStrikeItem = FALSE;                                    // not found item to strike yet 
                                                            
   if( m_pFont ) pfontOld = dc.SelectObject( m_pFont );    // select font
   dc.SetBkMode( TRANSPARENT );                            // draw only text

   // get text length
   uTextLength = ::SendMessage( GetSafeHwnd(), LB_GETTEXTLEN, pDIS->itemID, 0 );
   pszItemText = new char[uTextLength+1];                  // make a buffer 

   // get text
   ::SendMessage( GetSafeHwnd(), LB_GETTEXT, pDIS->itemID, (LPARAM)pszItemText );

   if( (m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE) &&
       (m_chStrikeClue == *pszItemText) )                  // is this a item to be striked out
   {                                                        
      colorref    = m_colStrikeItem;                       // strike color
      pszShowText = pszItemText + 1;                       // do not print the first character
      uTextLength--;

      size = dc.GetTextExtent( pszShowText, uTextLength ); // get text size
      bStrikeItem = TRUE;
   }
   else if( m_dwFlags & CTRLLBSTRIKEOUT_FLAGFORSTRIKE )
   {
      pszShowText = pszItemText;

      if( GetItemData( pDIS->itemID ) & CTRLLBSTRIKEOUT_STRIKE_FLAG )
      {
         colorref    = m_colStrikeItem;                      // strike color

         size = dc.GetTextExtent( pszShowText, uTextLength );// get text size
         bStrikeItem = TRUE;
      }
   }
   else                                                    // don't strike item
   {
      colorref    = m_colItem;
      pszShowText = pszItemText;
   }

   if( pDIS->itemState & ODS_SELECTED )                    // is item selected
   {
      // if unable to select item when striked, check if not striked
      if( (bStrikeItem == FALSE) ||                     
          !(m_dwFlags & CTRLLBSTRIKEOUT_UNABLESELECTSTRIKEITEM) )
      {
         brush.CreateSolidBrush( GetSysColor( COLOR_HIGHLIGHT ) );// create a higlight brush
         dc.FillRect( &pDIS->rcItem, &brush );             // draw rect with brush
         dc.DrawFocusRect( &pDIS->rcItem );                // draw a focus rect for selected item
         colorref = GetSysColor( COLOR_HIGHLIGHTTEXT );    // get text color for highlighted items
      }
   }
   else
   {
      brush.CreateSolidBrush( GetSysColor( COLOR_WINDOW ) );
      dc.FillRect( &pDIS->rcItem, &brush );
   }
 

   dc.SetTextColor( colorref );
   dc.TextOut( pDIS->rcItem.left+1, pDIS->rcItem.top+2, pszShowText, (int)uTextLength );

   if( bStrikeItem )                                       // draw a line across item
   {
      CPen  pen;
      CPen* ppenOld;                                       // old pen
      pen.CreatePen( PS_SOLID, 2, colorref );              // create a pen
      ppenOld = dc.SelectObject( &pen );

      // *** set the points for strike line ***
      point[0].x = pDIS->rcItem.left;
      point[0].y = 2 + pDIS->rcItem.top + (pDIS->rcItem.bottom - pDIS->rcItem.top) / 2;

      point[1].x = size.cx + 1;
      point[1].y = point[0].y;

      dc.Polyline( point, 2 );
      dc.SelectObject( ppenOld );
   }

   if( m_pFont ) dc.SelectObject( pfontOld );              // delete if selected a new font
   delete pszItemText;                                     // delete buffer for text
}

/*-------------------------------------------------------------------------------------------------
Name:    DeleteResource()
Type:    Function
Purpose: Delete resourses that have been created for this control

Param:   uResource = decides the resources that are to be deleted
*/
void CctrlLBStrikeOut::DeleteResource( UINT uResource /*=CTRLLBSTRIKEOUT_ALLRESOURCE*/ )
{
   ASSERT( uResource <= CTRLLBSTRIKEOUT_ALLRESOURCE );

   // *** Font ***
   if( uResource & CTRLLBSTRIKEOUT_FONT & m_dwFlags )
   {
      delete m_pFont;
      m_pFont = NULL;
   }

   // *** Normal color on each item ***
   if( uResource & CTRLLBSTRIKEOUT_ITEMCOLOR & m_dwFlags )
   {
      m_colItem = ::GetSysColor( COLOR_WINDOW );
   }

   // *** Color on all items that are disabled ***
   if( uResource & CTRLLBSTRIKEOUT_STRIKEITEMCOLOR & m_dwFlags )
   {
      m_colStrikeItem = ::GetSysColor( COLOR_WINDOW );
   }

   m_dwFlags &= ~(uResource);       // Clear flags for deleted resources
}


BEGIN_MESSAGE_MAP(CctrlLBStrikeOut, CListBox)
	//{{AFX_MSG_MAP(CctrlLBStrikeOut)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/************************************************
   *   Messages
   *
 * * *
  ***
   *
*/


/*-------------------------------------------------------------------------------------------------
Name:	   OnLBSOSelChange()
Type:    Message
Purpose: Special handling for selection changing
*/
void CctrlLBStrikeOut::OnLBSOSelChange()
{
   int   iSelectedItem;
   UINT  uTextLength;
   char* pszItemText;

   if( m_dwFlags & CTRLLBSTRIKEOUT_UNABLESELECTSTRIKEITEM )
   {
      iSelectedItem = GetCurSel();
      if( iSelectedItem != LB_ERR )
      {
         if( m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE )
         {
            // get text length
            uTextLength = ::SendMessage( GetSafeHwnd(), LB_GETTEXTLEN, iSelectedItem, 0 );
            pszItemText = new char[uTextLength+1];         // create buffer
            // get text
            ::SendMessage( GetSafeHwnd(), LB_GETTEXT, iSelectedItem, (LPARAM)pszItemText );
         
            if( *pszItemText == m_chStrikeClue )           // is item selected
            {
               FindNextSelectableItem( iSelectedItem );    // find next selectable item
            }
   
            delete pszItemText;

         }
         else if( m_dwFlags & CTRLLBSTRIKEOUT_FLAGFORSTRIKE )
         {
            if( GetItemData( iSelectedItem ) & CTRLLBSTRIKEOUT_STRIKE_FLAG )
            {
               FindNextSelectableItem( iSelectedItem );    // find next selectable item
            }
         }// endif
      }// endif
   }// endif

   m_iSelectedItem = iSelectedItem;                        // store index for selected item
   SetCurSel( m_iSelectedItem );                           // select item
}


/*-------------------------------------------------------------------------------------------------
Name:	   FindNextSelectableItem()
Type:    function (private)
Purpose: Finds next item that can be selected

Param:   iSelectedItem = item that not was able to be selected
*/
void CctrlLBStrikeOut::FindNextSelectableItem( int& iSelectedItem )
{
   ASSERT(m_dwFlags & CTRLLBSTRIKEOUT_UNABLESELECTSTRIKEITEM);
   ASSERT(iSelectedItem != m_iSelectedItem);
   ASSERT( (m_dwFlags & CTRLLBSTRIKEOUT_FLAGFORSTRIKE) || 
           (m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE) );

   int   iOffsetNextItem;     // Offset to next item 
   char* pszItemText = NULL;  // Pointer to buffer for text
   DWORD dwItemData;          // Itemdata

   // set offset
   iOffsetNextItem = ( (iSelectedItem - m_iSelectedItem) > 0 ) ? 1 : -1;

   while( iSelectedItem != -1 )
   {
      iSelectedItem += iOffsetNextItem;                    // next item
   
      if( m_dwFlags & CTRLLBSTRIKEOUT_CHARFORSTRIKE )      // char marks strike ?
      {
         if( pszItemText == NULL )                         // if buffer not created
         {
            pszItemText = new char[CTRLLBSTRIKEOUT_MAXTEXTSIZE];// create buffer
         }

         // get text
         if( ::SendMessage( GetSafeHwnd(), LB_GETTEXT, iSelectedItem, (LPARAM)pszItemText ) 
                == LB_ERR )
         {
            iSelectedItem = -1;                            // no item 
            break;
         }

         if( *pszItemText != m_chStrikeClue ) break;       // selectable item ?
      }
      else 
      {
         dwItemData = GetItemData( iSelectedItem );        // flag marks strike

         if( dwItemData == LB_ERR )                        // no item
         {
            iSelectedItem = -1;
            break;
         }
         
         // selectable item ?
         if( !(dwItemData & CTRLLBSTRIKEOUT_STRIKE_FLAG) ) break;
      }
   }

   // select previous item if not found a new selectable
   if( iSelectedItem == -1 ) iSelectedItem = m_iSelectedItem;
   // delete buffer if created
   if( pszItemText         ) delete pszItemText;
}



