/*
   FILNAME:       ctrlLBStrikeOut.h

   PURPOSE:       A control that has the ability to disable and draw a line across for items

   CREATED BY:    Per Ghosh
   ENVIRONMENT:	Visual C++


INFO:
*/

#ifndef __CTRLLBSTRIKEOUT_H_
#define __CTRLLBSTRIKEOUT_H_

// *** Resource ***
#define CTRLLBSTRIKEOUT_FONT                    0x00000001
#define CTRLLBSTRIKEOUT_ITEMCOLOR               0x00000002
#define CTRLLBSTRIKEOUT_STRIKEITEMCOLOR         0x00000004
#define CTRLLBSTRIKEOUT_ALLRESOURCE             0x00000007

// *** Behaviour ***
#define CTRLLBSTRIKEOUT_UNABLESELECTSTRIKEITEM  0x00010000
#define CTRLLBSTRIKEOUT_FLAGFORSTRIKE           0x00100000
#define CTRLLBSTRIKEOUT_CHARFORSTRIKE           0x00200000

// *** Status ***
#define CTRLLBSTRIKEOUT_STRIKE_FLAG             0x80000000

// *** Constant ***
#define CTRLLBSTRIKEOUT_MAXTEXTSIZE             100

class CctrlLBStrikeOut : public CListBox
{

// Construction
public:
	CctrlLBStrikeOut();
	virtual ~CctrlLBStrikeOut();

// Attributes
public:
   DWORD    m_dwFlags;                 // Status
   CFont*   m_pFont;                   // Font for text
   COLORREF m_colItem;                 // Color for items
   COLORREF m_colStrikeItem;           // Color for striked items

   char    m_chStrikeClue;             // sign that marks a striked item

   int     m_iSelectedItem;            // item that is selected

// Get/Set
public:
   int  GetSelectedItem() { return m_iSelectedItem; }

   void SetFont( CFont* pFont ) { m_pFont = pFont; }
	void SetStrikeClue( char chClue );
   void SetStrikeFlag( BOOL bFlag = TRUE );
   void SetColorForItem( COLORREF colorref ) { m_colItem = colorref; }
   void SetColorForStrikeItem( COLORREF colorref ) { m_colStrikeItem = colorref; }


// Operations
public:
	BOOL Create( DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT uId );
	void CreateDefaultResource();
	int  StrikeItem( UINT uItem, BOOL bStrike = TRUE );
	int  IsItemStriked( UINT uItem );
	void DeleteResource( UINT uResource = CTRLLBSTRIKEOUT_ALLRESOURCE );

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CctrlLBStrikeOut)
	public:
	virtual BOOL OnChildNotify(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pLResult);
	//}}AFX_VIRTUAL

// Implementation
private:
	void DrawItem(LPDRAWITEMSTRUCT pDIS);
   void OnLBSOSelChange();
	void FindNextSelectableItem( int& iSelectedItem );

	// Generated message map functions
protected:
	//{{AFX_MSG(CctrlLBStrikeOut)
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

#endif
